use super::get_command;
use serde::{Deserialize, Serialize};
use std::process::{Command, Stdio};

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct Workflow {
    pub name: String,
    pub inputs: Vec<String>,

    #[serde(rename = "step")]
    steps: Vec<WorkflowStep>,
}

pub type StringMap = std::collections::HashMap<String, String>;

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct WorkflowStep {
    r#type: String,

    #[serde(default)]
    values: StringMap,

    #[serde(default)]
    inputs: StringMap,

    #[serde(default)]
    outputs: StringMap,
}

impl Workflow {
    /// Generate a [`clap::Command`] for a [`Workflow`], where the inputs can be defined either by
    /// command-line arguments or via a JSON input file.
    pub fn generate_command(&self) -> clap::Command {
        let mut command = clap::Command::new(&self.name).arg(clap::arg!(
                --"input-file" [FILE]
                "A file containing any inputs not passed on the command line"
        ));
        for input in &self.inputs {
            let arg = clap::Arg::new(input)
                .required(false)
                .long(input)
                .value_name(input.to_uppercase());
            command = command.arg(arg);
        }
        command
    }

    fn load_inputs(&self, matches: &clap::ArgMatches) -> StringMap {
        let mut map = StringMap::default();
        let input_file: Option<StringMap> = matches
            .get_one::<std::path::PathBuf>("input-file")
            .and_then(|p| std::fs::File::open(p).ok())
            .and_then(|f| serde_json::from_reader(f).ok());
        for input in &self.inputs {
            match matches.get_one::<String>(&input.replace('_', "-")) {
                Some(value) => {
                    map.insert(input.clone(), value.clone());
                    continue;
                }
                None => {
                    if let Some(value) = input_file.as_ref().and_then(|f| f.get(input)) {
                        map.insert(input.clone(), value.clone());
                        continue;
                    }
                }
            }
            panic!("Key was not found: {input}");
        }

        map
    }

    pub fn handle(self, matches: &clap::ArgMatches) {
        let inputs = self.load_inputs(matches);
        // step 2: run through commands
    }
}
